# Code written by: 
# Juho Kopra / juho.j.kopra@jyu.fi / University of Jyväskylä, Finland / 20th April 2017

combine.mcmc.lists <- function(mcmc.samp1,mcmc.samp2) {
    l<-list(as.mcmc(rbind(mcmc.samp1[[1]],
	                  mcmc.samp2[[1]])
    ))
    as.mcmc.list(l)
}

# Define the length of burn-in, iterations, strata (number of temporary saves per chain) and thinning
n.burnin<-9000
n.iter <- 46900 #101699
n.strata <- 51 #113
n.thin <- 75

n.iter %% (n.thin*n.strata)
n.iter/(n.thin*n.strata)

# load initial values:
load("initialvalues.Rdata",verbose=TRUE)

initvalues$.RNG.seed<-id_rank
# initial values loaded
ls()

# load prerun data
load(file=paste("./runs/prerun_simul.Rdata",sep="",collapse=""),verbose=TRUE)
sel_variables<-c(sel_variables[sel_variables!="osal" & sel_variables!="" & sel_variables!="part"],"sumsmoke","sumnosmoke","pp","beta0","betaArea","betaAge","eta")


require(rjags)
time.initial<-system.time({
temp.model = jags.model(paste("./runs/",model_specific_folder,"/","jags_survival_model_smoking.txt",sep="",collapse=""), 
	            inits=initvalues[c("eta","pp","beta_cohort_birth","beta_cohort_smoke","betaSMOKE","betaAge",".RNG.name",".RNG.seed")],
	            data=dataList[names(dataList)!="age1"], n.chains=1, n.adapt=1000,quiet=FALSE)
})

time.updates<-rep(0,10)
time.coda<-rep(0,10)

time.burnin<-system.time(
	update(temp.model, n.iter=n.burnin)
)
time.coda[1]<-system.time(
	out.res<-coda.samples(temp.model, sel_variables, n.iter=n.iter/n.strata, thin=n.thin,progress.bar="text")
)[3]
save(out.res,time.updates,time.burnin,time.coda,time.initial, file=paste("./runs/",model_specific_folder,"/images/image",1,"chain",id_rank,".Rdata",sep=""))

for (i in 2:n.strata) {
	time.coda[i]<-system.time(
	    out.tmp<-coda.samples(temp.model, sel_variables, n.iter=n.iter/n.strata, thin=n.thin,progress.bar="text")
	)[3]
	out.res<-combine.mcmc.lists(out.res,out.tmp)
	save(out.res,time.updates,time.burnin,time.coda,time.initial, file=paste("./runs/",model_specific_folder,"/images/image",i,"chain",id_rank,".Rdata",sep=""))
}
out.res

save.image(file=paste("./runs/",model_specific_folder,"/","mcmc_fit.Rdata",sep="",collapse=""))
