# Code written by: 
# Juho Kopra / juho.j.kopra@jyu.fi / University of Jyväskylä, Finland / 20th April 2017

require(R2jags)
source("./codes/regex_model_pars_from_string.r")

# DATA:
load(file="./datas/frdata_for_JAGS_20042017.Rdata",verbose=TRUE)
frdata<-subset(frdata,subset=c(year<=2007 & age1>age0 & age0 <=65 & (area<=3 | is.na(area))))
narea<-length(table(frdata$area))

frdata$event<-as.integer(frdata$event>0)

# covariates for participation sub-model P(M|x,Y) excluding Y. Baseline level is men, year 1972, 45-years-old
part=as.integer(!is.na(frdata$smoking)) #participation indicator
woman=frdata$sex-1
year=1+(frdata$year-1972)/5
age0=frdata$age0-45
area=frdata$area

upperlimit<-floor(max(frdata[,1:3])+1)
frdata$age0r<-cut(frdata$age0,breaks = seq(25,upperlimit,1))
frdata$age1r<-cut(frdata$age1,breaks = seq(25,upperlimit,1))

### SINGLE IMPUTATION FOR REGION ###
prob_area2<-c(0.495, 0.493)
loop<-which(is.na(frdata$area)==TRUE)
imput_area<-rbinom(length(loop),size=1,prob=prob_area2[1+(frdata$year[loop]-1972)/5]) + 2
frdata$area[loop]<-imput_area
##########################################

# compute dN[i,j]
# 1) function to calculate the dN-object (how many persons died during one-year -period defined by "breaks")
f1<-function(dat,breaks) {
    dat.age<-dat$age1
    dat.event<-dat$event
    res<-matrix(0,ncol=length(breaks)-1,nrow=dim(dat)[1])
    for(i in 1:length(dat.age)) {
      res[i,pmin(max(breaks),floor(dat.age[i]))-min(breaks)-1]<-dat.event[i]
    }
    res
}
dN<-f1(dat=frdata,breaks=seq(25,upperlimit-1))

# 2) function to calculate the number of people "under the risk" = alive before the start of one-year -period
f2<-function(dat,breaks) {
    dat.age<-dat$age1
    res<-matrix(0,ncol=length(breaks)-1,nrow=dim(dat)[1])
    for(i in 1:length(dat.age)) {
        res[i,1:(pmin(max(breaks),floor(dat.age[i]))-min(breaks))]<-1
    }
    res
}
alive0<-f2(dat=frdata,breaks=seq(25,upperlimit-1))
plot(25:98,colSums(dN) / colSums(alive0))


# 3) smoking vector (SMOKE stands for smoking indicator)
SMOKE<-as.integer(as.numeric(frdata$smoking)==3)

# 4) i.sex ja i.area
i.sex  <- frdata$sex
i.area <- frdata$area

# 5) birthcohortSMOKE
birthcohortSMOKE<-frdata$year-frdata$age0

# Ntotal
Ntotal<-dim(dN)[1]

# 6) pick2
yearSMOKE_part<-frdata$year
sexSMOKE_part<-frdata$sex
areaSMOKE_part<-frdata$area
nyears<-length(table(frdata$year))
#nareas<-length(table(frdata$area))
pick1_part<-(yearSMOKE_part-1972)/nyears+1+(sexSMOKE_part-1)*nyears+(areaSMOKE_part-2)*2*nyears
if (any(rowSums((table(pick1_part,yearSMOKE_part,sexSMOKE_part,areaSMOKE_part)>0)+0) != 1)) { print("Mallissa X->Y on virhe!"); }
names(pick1_part)<-paste(sexSMOKE_part,"&",yearSMOKE_part,"&",areaSMOKE_part)
pick1df_part<-merge(table(pick1_part),table(names(pick1_part)))
label_pick2_part<-as.character(pick1df_part$Var1) # tämä on parametrien nimet
numbers_part<-sort(unique(pick1_part))
numbers2_part<-1:length(numbers_part)
names(numbers2_part)<-numbers_part
pick2_part<-numbers2_part[paste(pick1_part)]
names(pick2_part)<-names(pick1_part)
pick2<-pick2_part
rm(list=c("yearSMOKE_part","pick1_part","pick2_part","pick1df_part","numbers_part","numbers2_part"))
table(pick2,useNA="always")
table(pick2,frdata$sex,useNA="always")

selm<-frdata$year<=1977 & frdata$sex==1
seln<-frdata$year<=1977 & frdata$sex==2
table(pick2[selm],frdata$year[selm],frdata$area[selm],useNA="always")
table(pick2[seln],frdata$year[seln],frdata$area[seln],useNA="always")

# dNalt
dNalt<-array(NA,dim = c(2,narea,2,dim(dN)[2]))
alive0alt<-array(NA,dim = c(2,narea,2,dim(dN)[2]))

for(r in c(1:dim(dN)[2])) {
    for(s in 1:2) {
        for(a in 1:narea) {
            for(t in 1:2) {
                dNalt[s,a,t,r]<-sum(dN[i.sex==s & i.area==a+1 & SMOKE==t-1 & !is.na(SMOKE),r])
                alive0alt[s,a,t,r]<-sum(alive0[i.sex==s & i.area==a+1 & SMOKE==t-1 & !is.na(SMOKE),r])
            }
        }
    }
}
rowcumSums<-function(x) {t(apply(x,1,cumsum));}
rowcumSums(dNalt[1,,1,])
rowcumSums(dNalt[1,,2,])
rowcumSums(dNalt[2,,1,])
rowcumSums(dNalt[2,,2,])

cutUpperEndZeros<-function(dat,index=TRUE) {
    sel<-which(dat==0)
    sel<- sel[(which.max(diff(sel))+1):length(sel)]
    if(index) {
        return(sel)
    }
    dat[-sel]
}

rivi<-1:narea
step<-0.00
mn<-c("men","women")
par(mfrow=c(2,1))
for(s in 1:2) {
    dat<-colSums(dNalt[s,rivi,1,]+step)/colSums(alive0alt[s,rivi,1,]+step)
    dat[is.nan(dat)]<-0
    x<-1:length(dat)
    dat2<-colSums(dNalt[s,rivi,2,]+step)/colSums(alive0alt[s,rivi,2,]+step)
    dat2[is.nan(dat2)]<-0
    x2<-1:length(dat2)
    plot(dat[x],main=mn[s],ylim=range(c(dat,dat2)))
    points(dat2[x2],col="red")
    
    sel<-cutUpperEndZeros(dat[x])
    loe<-lowess(dat[-sel] ~ x[-sel])
    lines(loe)
    sel2<-cutUpperEndZeros(dat2[x2])
    
    fit<-glm(dat[-sel] ~ x[-sel],family="quasibinomial")
    expit<-function(x) {exp(x)/(1+exp(x));}
    pred<-expit(cbind(1,x2) %*% coef(fit))
    lines(x2,pred)
    cat("c(",paste(pred,collapse=", "),")\n",sep=" ") # We use these in initial values to make MCMC chains to start.
    
    loe2<-lowess(dat2[-sel2] ~ x2[-sel2])
    lines(loe2,col="red")
}


alive0alt[1,,1,]
alive0alt[1,,2,]
alive0alt[2,,1,]
alive0alt[2,,2,]

# dN_np
dN_np<-dN[is.na(SMOKE),]
dim(dN_np)

# alive0_np
alive0_np<-alive0[is.na(SMOKE),]
dim(alive0_np)

# SMOKE_np
SMOKE_np<-SMOKE[is.na(SMOKE)]
length(SMOKE_np)

# i.sex_np
i.sex_np<-i.sex[is.na(SMOKE)]
length(i.sex_np)

# i.area_np
i.area_np<-i.area[is.na(SMOKE)]
length(i.area_np)

# Ntotal_np
Ntotal_np <- sum(is.na(SMOKE))

# inp
inp<-c(1:Ntotal)[is.na(SMOKE)]

model <- "
# JAGS model written by: Juho Kopra / juho.j.kopra@jyu.fi / University of Jyväskylä, Finland / 20th April 2017
model
{
    # survival model
    for (j in 1:T) {
	      # participants
        for(s in 1:2) {
            for(a in 1:narea) {
                # no smokers
                dNalt[s,a,1,j] ~ dpois(theta[s,a,1,j])
                theta[s,a,1,j] <- alive0alt[s,a,1,j]*exp(betaSMOKE[s]*0)*pp[s,j] # + 0.001
                
                # smokers
                dNalt[s,a,2,j] ~ dpois(theta[s,a,2,j])
                theta[s,a,2,j] <- alive0alt[s,a,2,j]*exp(betaSMOKE[s]*1)*pp[s,j] # + 0.001
                
            }
        }
        # non-participants
        for (i in 1:Ntotal_np) {
            #inp[i] is index for non-participants among all subjects. Use this only for SMOKE and when looping over non-participants only!
            dN_np[i,j] ~ dpois(theta_np[i,j])
            theta_np[i,j] <- alive0_np[i,j]*exp(betaSMOKE[i.sex_np[i]]*SMOKE[inp[i]])*pp[i.sex_np[i],j]
        }
    }

    # logistic regression models
    for (ii in 1:Ntotal) {
        # logistic regression model for  X-> Y (this gives a prior for non-participants smoking)
	      # risk factor sub-model
        SMOKE[ii] ~ dbern(psmoke[ii])
        logit(psmoke[ii]) <- beta_cohort_birth[pick2[ii]] + (birthcohortSMOKE[ii]-1938)*beta_cohort_smoke[pick2[ii]]
    
	      # participation sub-model
        part[ii] ~ dbern(ppart[ii])
        logit(ppart[ii]) <- beta0[sex[ii],year[ii]] + eta[sex[ii],year[ii]]*SMOKE[ii] + betaAge[sex[ii],SMOKE[ii]+1]*age0[ii] + betaArea[area[ii]]
    }

    # prior distributions for participation model:
    for (iarea in 2:narea) {
        betaArea[iarea] ~ dnorm(0,0.001)
    }
    betaArea[1] <- 0.0

    for (isex in 1:nsex) {
  		for (iyear in 1:nyear) {
  			beta0[isex,iyear] ~ dnorm(0,0.001)
  			eta[isex,iyear] ~ dlogis(0,2.05) # tau is a precision parameter = 1/variance
  		}
  		betaAge[isex,1] ~ dnorm(0,0.001)
		  betaAge[isex,2] ~ dnorm(0,0.001)
    }
    # prior distribution for how smoking affects risk of disease
    betaSMOKE[1] ~ dnorm(0,0.001)
    betaSMOKE[2] ~ dnorm(0,0.001)
    
    # both gender have their own baseline
    for (i.sex2 in 1:nsex) {
        for (j2 in 2:T) {
            pp[i.sex2,j2] ~ dunif(pp[i.sex2,j2-1],20)
        }
        pp[i.sex2,1] ~ dunif(0,20)
    }

    for(k in 1:npick2) {
        beta_cohort_birth[k] ~ dnorm(0,0.001)
        beta_cohort_smoke[k] ~ dnorm(0,0.001)

        sumsmoke[k] <- sum(SMOKE[1:Ntotal]*equals(pick2[1:Ntotal],k))
        sumnosmoke[k] <- sum((1-SMOKE[1:Ntotal])*equals(pick2[1:Ntotal],k))
    }
}"
cat(model,file=paste("./runs/",model_specific_folder,"/","jags_survival_model_smoking.txt",sep="",collapse=""))

sel_variables<-stripVariablesFromModelFile(model)
sel_variables<-setdiff(sel_variables,c("zeros","SMOKE","PART","zeros_np","PART_np"))
sel_variables

dataList = list(
  dNalt=dNalt,
  alive0alt=alive0alt,
  SMOKE=SMOKE,
  birthcohortSMOKE=birthcohortSMOKE,
  i.sex=i.sex,
  i.area=i.area,
  nsex=2,
  pick2 = pick2,
  npick2 = length(unique(pick2)),
  T = dim(dN)[2],
  Ntotal=Ntotal,
  
  dN_np=dN_np,
  alive0_np=alive0_np,
  SMOKE_np=SMOKE_np,
  i.sex_np=i.sex_np,
  i.area_np=i.area_np,
  inp=inp,
  Ntotal_np=Ntotal_np,

  part=part,
  woman=woman,
  year=year,
  age0=age0,
  age1=frdata$age1,
  age2=frdata$age2,
  
  nyear=length(unique(year)),
  narea=narea,
  sex=sexSMOKE_part,
  area=areaSMOKE_part-1
)

inits<-function() {
  list()
}

ls()
rm(list=setdiff(ls(),c("inits","dataList","par.trace.samples","combine.samples","sel_variables",never_rm,"label_pick2","label_pick2_np")))
ls()
save(list=ls(),file=paste("./runs/prerun.Rdata",sep="",collapse=""))


